import Filesystem.*;

public class Server extends Ice.Application
{
    public int
    run(String[] args)
    {
        // Shut down cleanly on interrupt.
        //
        shutdownOnInterrupt();

        Ice.ObjectAdapter adapter = communicator().createObjectAdapterWithEndpoints("Lab2", "default -p 10000");

        // Create the root directory (with name "/" and no parent).
        //
        DirectoryI root = new DirectoryI("/", null);
        root.activate(adapter);

        // Create a file called "README" in the root directory.
        //
        FileI file = new FileI("README", root);
        String[] text = new String[] { "This file system contains a collection of poetry." };
        try
        {
            file.write(text);
        }
        catch(IOError e)
        {
            // Implementation won't throw.
        }
        file.activate(adapter);

        // Create a directory called "Coleridge" in the root directory.
        //
        DirectoryI coleridge = new DirectoryI("Coleridge", root);
        coleridge.activate(adapter);

        // Create a file called "Kubla_Khan" in the Coleridge directory.
        //
        file = new FileI("Kubla_Khan", coleridge);
        text = new String[]{ "In Xanadu did Kubla Khan",
                             "A stately pleasure-dome decree:",
                             "Where Alph, the sacred river, ran",
                             "Through caverns measureless to man",
                             "Down to a sunless sea." };
        try
        {
            file.write(text);
        }
        catch(IOError e)
        {
            // Implementation won't throw.
        }
        file.activate(adapter);

        // All objects are created, allow client requests now.
        //
        adapter.activate();

        // Wait until we are done.
        //
        communicator().waitForShutdown();
        if(interrupted())
        {
            System.out.println(appName() + ": received signal, shutting down");
        }

        return 0;
    }

    public static void
    main(String[] args)
    {
        Server app = new Server();
        int status = app.main("Server", args);
        System.exit(status);
    }
}
