import Filesystem.*;

public class Client
{
    static void
    listRecursive(DirectoryPrx dir, int depth)
    {
        char[] indentCh = new char[++depth];
        java.util.Arrays.fill(indentCh, '\t');
        String indent = new String(indentCh);

        NodeDetails[] contents = dir.list();

        for(int i = 0; i < contents.length; ++i)
        {
            System.out.println(indent + contents[i].name + "(" +
                               (contents[i].type == NodeType.DirT ? "directory" : "file") + "):");
            if(contents[i].type == NodeType.DirT)
            {
                DirectoryPrx subdir = DirectoryPrxHelper.uncheckedCast(contents[i].proxy);
                listRecursive(subdir, depth);
            }
            else
            {
                FilePrx file = FilePrxHelper.uncheckedCast(contents[i].proxy);
                boolean error = true;
                do
                {
                    try
                    {
                        String[] text = file.read();
                        for(int j = 0; j < text.length; ++j)
                        {
                            System.out.println(indent + "\t" + text[j]);
                        }
                        error = false;
                    }
                    catch(IOError e)
                    {
                        System.out.println("Caught IO error: " + e.reason + " Retrying...");
                    }
                }
                while(error);
            }
        }
    }

    public static void
    main(String[] args)
    {
        int status = 0;
        Ice.Communicator ic = null;
        try
        {
            // Create a communicator
            //
            ic = Ice.Util.initialize(args);

            // Create a proxy for the root directory
            //
            Ice.ObjectPrx base = ic.stringToProxy("RootDir:default -p 10000");

            // Down-cast the proxy to a Directory proxy
            //
            DirectoryPrx rootDir = DirectoryPrxHelper.checkedCast(base);
            if(rootDir == null)
                throw new RuntimeException("Invalid proxy");

            // Recursively list the contents of the root directory
            //
            System.out.println("Contents of root directory:");
            listRecursive(rootDir, 0);
        }
        catch(Ice.LocalException e)
        {
            e.printStackTrace();
            status = 1;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            status = 1;
        }
        if(ic != null) {
            // Clean up
            //
            try
            {
                ic.destroy();
            }
            catch(Exception e)
            {
                e.printStackTrace();
                status = 1;
            }
        }
        System.exit(status);
    }
}
