import Filesystem.*;

public class Client extends Ice.Application
{
    static void
    listRecursive(DirectoryPrx dir, int depth)
    {
        char[] indentCh = new char[++depth];
        java.util.Arrays.fill(indentCh, '\t');
        String indent = new String(indentCh);

        NodeDetails[] contents = dir.list();

        for(int i = 0; i < contents.length; ++i)
        {
            System.out.print(indent + contents[i].name + "(" +
                             (contents[i].type == NodeType.DirT ? "directory" : "file"));
            if(_showSize)
            {
                System.out.print(", " + contents[i].proxy.size());
            }
            System.out.println("):");
            if(contents[i].type == NodeType.DirT)
            {
                DirectoryPrx subdir = DirectoryPrxHelper.uncheckedCast(contents[i].proxy);
                listRecursive(subdir, depth);
            }
            else
            {
                FilePrx file = FilePrxHelper.uncheckedCast(contents[i].proxy);
                boolean error = true;
                do
                {
                    try
                    {
                        String[] text = file.read();
                        for(int j = 0; j < text.length; ++j)
                        {
                            System.out.println(indent + "\t" + text[j]);
                        }
                        error = false;
                    }
                    catch(IOError e)
                    {
                        System.out.println("Caught IO error: " + e.reason + " Retrying...");
                    }
                }
                while(error);
            }
        }
    }

    public int run(String[] args)
    {
        // Create a proxy for the root directory
        //
        Ice.ObjectPrx base = communicator().propertyToProxy("Filesystem.RootDir");

        // Down-cast the proxy to a Directory proxy
        //
        DirectoryPrx rootDir = DirectoryPrxHelper.checkedCast(base);
        if(rootDir == null)
            throw new RuntimeException("Invalid proxy");

        _showSize = communicator().getProperties().getPropertyAsInt("Filesystem.ShowSize") != 0;

        // Recursively list the contents of the root directory
        //
        System.out.println("Contents of root directory:");
        listRecursive(rootDir, 0);
        return 0;
    }


    public static void
    main(String[] args)
    {
        Ice.InitializationData initData = new Ice.InitializationData();
        initData.properties = Ice.Util.createProperties();
        args = initData.properties.parseCommandLineOptions("Filesystem", args);
        Client app = new Client();
        int status = app.main("Client", args, initData);
        System.exit(status);
    }

    static boolean _showSize;
}
