import Filesystem.*;

public class ListThread extends Thread
{
    public
    ListThread(DirectoryPrx dir, String name)
    {
        super(name);
        _dir = dir;
    }

    public void
    run()
    {
        listRecursive(_dir, 0);
    }

    private static void
    listRecursive(DirectoryPrx dir, int depth)
    {
        char[] indentCh = new char[++depth];
        java.util.Arrays.fill(indentCh, '\t');
        String indent = new String(indentCh);

        NodeDetails[] contents = dir.list();

        for(int i = 0; i < contents.length; ++i)
        {
            System.out.println(indent + contents[i].name + "(" +
                               (contents[i].type == NodeType.DirT ? "directory" : "file") + "):");
            if(contents[i].type == NodeType.DirT)
            {
                DirectoryPrx subdir = DirectoryPrxHelper.uncheckedCast(contents[i].proxy);
                listRecursive(subdir, depth);
            }
            else
            {
                FilePrx file = FilePrxHelper.uncheckedCast(contents[i].proxy);
                boolean error = true;
                do
                {
                    try
                    {
                        String[] text = file.read();
                        for(int j = 0; j < text.length; ++j)
                        {
                            System.out.println(indent + "\t" + text[j]);
                        }
                        error = false;
                    }
                    catch(IOError e)
                    {
                        System.out.println("Caught IO error: " + e.reason + " Retrying...");
                    }
                }
                while(error);
            }
        }
    }

    private DirectoryPrx _dir;
}
