package Filesystem;

public final class DirectoryI extends _DirectoryDisp implements NodeI
{
    public
    DirectoryI(String name, DirectoryI parent)
    {
        _name = name;
        _parent = parent;
        _myID = parent != null ? java.util.UUID.randomUUID().toString() : "RootDir";
    }

    public String
    id()
    {
        return _myID;
    }

    public void
    activate(Ice.ObjectAdapter adapter)
    {
        Ice.Identity id = new Ice.Identity();
        id.name = _myID;
        adapter.add(this, id);
        if(_parent != null)
        {
            _parent.addChild(_name, this);
        }
    }

    public String
    name(Ice.Current c)
    {
        return _name;
    }

    public NodeDetails[]
    list(Ice.Current c)
    {
	Thread t = Thread.currentThread();

        System.out.println(t.getName() + " entering list");

        NodeDetails[] s;
        synchronized(this)
        {
            while(true)
            {
                try
                {
                    t.sleep(1000);
                    break;
                }
                catch(InterruptedException ex)
                {
                    // Ignored
                }
            }

            s = new NodeDetails[_contents.size()];
            int i = 0;
            for(java.util.Map.Entry<String, NodeI> e : _contents.entrySet())
            {
                NodeI val = e.getValue();
                Ice.Identity id = new Ice.Identity();
                id.name = val.id();
                s[i] = new NodeDetails();
                s[i].type = val instanceof DirectoryI ? NodeType.DirT : NodeType.FileT;
                s[i].proxy = NodePrxHelper.uncheckedCast(c.adapter.createProxy(id));
                s[i].name = s[i].proxy.name();
                ++i;
            }
        }

        System.out.println(t.getName() + " leaving list");

        return s;
    }

    public synchronized void
    addChild(String name, NodeI child)
    {
        _contents.put(name, child);
    }

    private String _name;
    private DirectoryI _parent;
    private String _myID;
    private java.util.HashMap<String, NodeI> _contents = new java.util.HashMap<String, NodeI>();
}
