import Filesystem.*;

public class Client extends Ice.Application
{
    static void
    listRecursive(DirectoryPrx dir, int depth)
    {
        char[] indentCh = new char[++depth];
        java.util.Arrays.fill(indentCh, '\t');
        String indent = new String(indentCh);

        NodeDetails[] contents = dir.list();

        for(int i = 0; i < contents.length; ++i)
        {
            System.out.println(indent + contents[i].name + "(" +
                               (contents[i].type == NodeType.DirT ? "directory" : "file") + "):");
            if(contents[i].type == NodeType.DirT)
            {
                DirectoryPrx subdir = DirectoryPrxHelper.uncheckedCast(contents[i].proxy);
                listRecursive(subdir, depth);
            }
            else
            {
                FilePrx file = FilePrxHelper.uncheckedCast(contents[i].proxy);
                boolean error = true;
                do
                {
                    try
                    {
                        String[] text = file.read();
                        for(int j = 0; j < text.length; ++j)
                        {
                            System.out.println(indent + "\t" + text[j]);
                        }
                        error = false;
                    }
                    catch(IOError e)
                    {
                        System.out.println("Caught IO error: " + e.reason + " Retrying...");
                    }
                }
                while(error);
            }
        }
    }

    public int run(String[] args)
    {
        // Create a proxy for the root directory
        //
        Ice.ObjectPrx base = communicator().stringToProxy("RootDir:default -p 10000");

        // Down-cast the proxy to a Directory proxy
        //
        DirectoryPrx rootDir = DirectoryPrxHelper.checkedCast(base);
        if(rootDir == null)
            throw new RuntimeException("Invalid proxy");

        FilePrx readme = null;
        try
        {
            readme = rootDir.createFile("README");
        }
        catch(NameInUse ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }
        String[] text = new String[] { "This file system contains a collection of poetry." };
        try
        {
            readme.write(text);
        }
        catch(IOError ex)
        {
            // Ignore, server won't throw this.
        }

        try
        {
            rootDir.createFile("README");
            Thread.dumpStack();
            System.exit(1);
        }
        catch(NameInUse ex)
        {
            System.out.println("Cannot create the same file twice, got NameInUse, as expected");
        }

        DirectoryPrx coleridge = null;
        try
        {
            coleridge = rootDir.createDir("Coleridge");
        }
        catch(NameInUse ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            rootDir.createDir("Coleridge");
            Thread.dumpStack();
            System.exit(1);
        }
        catch(NameInUse ex)
        {
            System.out.println("Cannot create the same directory twice, got NameInUse, as expected");
        }

        FilePrx khubla = null;
        try
        {
            khubla = coleridge.createFile("Khubla_Khan");
            text = new String[]{ "In Xanadu did Kubla Khan",
                                 "A stately pleasure-dome decree:",
                                 "Where Alph, the sacred river, ran",
                                 "Through caverns measureless to man",
                                 "Down to a sunless sea." };
            khubla.write(text);
        }
        catch(NameInUse ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }
        catch(IOError ex)
        {
            // Ignore, server won't throw this.
        }

        // Recursively list the contents of the root directory
        //
        System.out.println("Contents of root directory:");
        listRecursive(rootDir, 0);

/* PART_2
        try
        {
            coleridge.destroy();
            Thread.dumpStack();
            System.exit(1);
        }
        catch(DirNotEmpty ex)
        {
            System.out.println("Cannot destroy non-empty directory, got DirNotEmpty, as expected");
        }

        try
        {
            khubla.destroy();
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            khubla.destroy();
            Thread.dumpStack();
            System.exit(1);
        }
        catch(Ice.ObjectNotExistException ex)
        {
            System.out.println("Cannot destroy directory twice, got ObjectNotExistException, as expected");
        }
        catch(DirNotEmpty ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            readme.destroy();
            coleridge.destroy();
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            readme.destroy();
            Thread.dumpStack();
            System.exit(1);
        }
        catch(Ice.ObjectNotExistException ex)
        {
            System.out.println("Cannot destroy file twice, got ObjectNotExistException, as expected");
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            rootDir.destroy();
            Thread.dumpStack();
            System.exit(1);
        }
        catch(DirNotEmpty ex)
        {
            System.out.println("Cannot destroy root directory, got DirNotEmpty, as expected");
        }

        NodeDetails[] contents = rootDir.list();
        if(contents.length != 0)
        {
            Thread.dumpStack();
            System.exit(1);
        }
PART_2 */

        return 0;
    }

    public static void
    main(String[] args)
    {
        Client app = new Client();
        int status = app.main("Client", args);
        System.exit(status);
    }
}
