package Filesystem;

public class FileI extends _FileDisp
{
    public
    FileI(String name, DirectoryI parent, Ice.ObjectAdapter adapter)
    {
        assert(parent != null);

        _name = name;
        _myID = java.util.UUID.randomUUID().toString();
        _parent = parent;
        parent.addChild(_myID, this);
        adapter.add(this, adapter.getCommunicator().stringToIdentity(_myID));
    }

    public synchronized String
    name(Ice.Current c)
    {
        checkDestroyed();

        return _name;
    }

    public void
    destroy(Ice.Current c)
    {
        synchronized(this)
        {
            checkDestroyed();
            _destroyed = true;
        }

        c.adapter.remove(c.id);
        _parent.removeChild(_myID);
    }

    public synchronized String[]
    read(Ice.Current c)
        throws IOError
    {
        checkDestroyed();

        return _lines;
    }

    public synchronized void
    write(String[] text, Ice.Current c)
        throws IOError
    {
        checkDestroyed();

        _lines = text;
    }

    //
    // Helper functions. Throws if _destroyed is set.
    // Must be called with this servant locked.
    //
    private void
    checkDestroyed()
        throws Ice.ObjectNotExistException
    {
        if(_destroyed)
        {
            throw new Ice.ObjectNotExistException();
        }
    }

    public String _name; // Public for use by _parent
    public String _myID; // Public for use by _parent

    private DirectoryI _parent;
    private String[] _lines;
    private boolean _destroyed = false;
}
