import Filesystem.*;

public class Client extends Glacier2.Application
{
    static void
    listRecursive(DirectoryPrx dir, int depth)
    {
        char[] indentCh = new char[++depth];
        java.util.Arrays.fill(indentCh, '\t');
        String indent = new String(indentCh);

        NodeDetails[] contents = dir.list();

        for(int i = 0; i < contents.length; ++i)
        {
            System.out.println(indent + contents[i].name + "(" +
                               (contents[i].type == NodeType.DirT ? "directory" : "file") + "):");
            if(contents[i].type == NodeType.DirT)
            {
                DirectoryPrx subdir = DirectoryPrxHelper.uncheckedCast(contents[i].proxy);
                listRecursive(subdir, depth);
            }
            else
            {
                FilePrx file = FilePrxHelper.uncheckedCast(contents[i].proxy);
                boolean error = true;
                do
                {
                    try
                    {
                        String[] text = file.read();
                        for(int j = 0; j < text.length; ++j)
                        {
                            System.out.println(indent + "\t" + text[j]);
                        }
                        error = false;
                    }
                    catch(IOError e)
                    {
                        System.out.println("Caught IO error: " + e.reason + " Retrying...");
                    }
                }
                while(error);
            }
        }
    }

    public Glacier2.SessionPrx createSession()
    {
        String username = "joe";
        String password = "joe";
        Glacier2.SessionPrx session = null;
        try
        {
            session = router().createSession(username, password);
        }
        catch(Glacier2.PermissionDeniedException ex)
        {
            System.err.println("cannot create session: " + ex.reason);
            ex.printStackTrace();
            System.exit(1);
        }
        catch(Glacier2.CannotCreateSessionException ex)
        {
            System.err.println("cannot create session: " + ex.reason);
            ex.printStackTrace();
            System.exit(1);
        }
        return session;
    }

    public int runWithSession(String[] args)
        throws RestartSessionException
    {
        // Create a proxy for the root directory
        //
        Ice.ObjectPrx base = communicator().stringToProxy("RootDir:default -p 10000");

        // Down-cast the proxy to a Directory proxy
        //
        DirectoryPrx rootDir = DirectoryPrxHelper.checkedCast(base);
        if(rootDir == null)
            throw new RuntimeException("Invalid proxy");

        try
        {
            FilePrx readme = rootDir.createFile("README");
            String[] text = new String[] { "This file system contains a collection of poetry." };
            readme.write(text);

            DirectoryPrx coleridge = rootDir.createDir("Coleridge");

            FilePrx khubla = coleridge.createFile("Khubla_Khan");
            text = new String[]{ "In Xanadu did Kubla Khan",
                                 "A stately pleasure-dome decree:",
                                 "Where Alph, the sacred river, ran",
                                 "Through caverns measureless to man",
                                 "Down to a sunless sea." };
            khubla.write(text);

            System.out.println("Contents of root directory:");
            listRecursive(rootDir, 0);

            khubla.destroy();
            readme.destroy();
            coleridge.destroy();
        }
        catch(Ice.UserException ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        return 0;
    }

    public static void
    main(String[] args)
    {
        Client app = new Client();
        int status = app.main("Client", args);
        System.exit(status);
    }
}
