package Filesystem;

public final class DirectoryI extends _DirectoryDisp
{
    public static void
    makeRootDir(Ice.ObjectAdapter adapter)
    {
        DirectoryI root = new DirectoryI();
        root._name = "/";
        root._parent = null;
        root._myID = "RootDir";
        adapter.add(root, adapter.getCommunicator().stringToIdentity(root._myID));
    }

    private
    DirectoryI()
    {
    }

    public
    DirectoryI(String name, DirectoryI parent, Ice.ObjectAdapter adapter)
    {
        assert(parent != null);

        _name = name;
        _parent = parent;
        _myID = java.util.UUID.randomUUID().toString();
        parent.addChild(_myID, this);
        adapter.add(this, adapter.getCommunicator().stringToIdentity(_myID));
    }

    public synchronized String
    name(Ice.Current c)
    {
        checkDestroyed();

        return _name;
    }

    public synchronized FilePrx
    createFile(String name, Ice.Current c)
        throws NameInUse
    {
        checkDestroyed();

        checkNameInUse(name);
        FileI f = new FileI(name, this, c.adapter);
        return FilePrxHelper.uncheckedCast(c.adapter.createProxy(
            c.adapter.getCommunicator().stringToIdentity(f._myID)));
    }

    public synchronized DirectoryPrx
    createDir(String name, Ice.Current c)
        throws NameInUse
    {
        checkDestroyed();

        checkNameInUse(name);
        DirectoryI d = new DirectoryI(name, this, c.adapter);
        return DirectoryPrxHelper.uncheckedCast(c.adapter.createProxy(
            c.adapter.getCommunicator().stringToIdentity(d._myID)));
    }

    public void
    destroy(Ice.Current c)
        throws DirNotEmpty
    {
        synchronized(this)
        {
            checkDestroyed();
            if(!_contents.isEmpty())
            {
                throw new DirNotEmpty();
            }
            if(c.id.name.equals("RootDir"))
            {
                throw new DirNotEmpty();
            }
            _destroyed = true;
        }

        c.adapter.remove(c.id);
        _parent.removeChild(_myID);
    }

    public synchronized NodeDetails[]
    list(Ice.Current c)
    {
        checkDestroyed();

        java.util.List<NodeDetails> l = new java.util.ArrayList<NodeDetails>();
        for(java.util.Map.Entry<String, Node> e : _contents.entrySet())
        {
            Ice.Identity id = new Ice.Identity();
            id.name = e.getKey();
            Node val = e.getValue();
            NodeDetails details = new NodeDetails();
            details.type = val instanceof DirectoryI ? NodeType.DirT : NodeType.FileT;
            details.proxy = NodePrxHelper.uncheckedCast(c.adapter.createProxy(id));
            try
            {
                details.name = details.proxy.name();
                l.add(details);
            }
            catch(Ice.ObjectNotExistException ex)
            {
                // Ignore
            }
        }

        return l.toArray(new NodeDetails[0]);
    }

    public synchronized void
    addChild(String id, Node child)
    {
        _contents.put(id, child);
    }

    public synchronized void
    removeChild(String id)
    {
        _contents.remove(id);
    }

    //
    // Helper function. Throws if name is already in use.
    // Must be called with this servant locked.
    //
    private void
    checkNameInUse(String name)
        throws NameInUse
    {
        for(java.util.Map.Entry<String, Node> e : _contents.entrySet())
        {
            Node val = e.getValue();
            if(val instanceof DirectoryI)
            {
                if(((DirectoryI)val)._name.equals(name))
                {
                    throw new NameInUse();
                }
            }
            else
            {
                if(((FileI)val)._name.equals(name))
                {
                    throw new NameInUse();
                }
            }
        }
    }

    //
    // Helper functions. Throws if _destroyed is set.
    // Must be called with this servant locked.
    //
    private void
    checkDestroyed()
    {
        if(_destroyed)
        {
            throw new Ice.ObjectNotExistException();
        }
    }

    private String _name;
    private DirectoryI _parent;
    private String _myID;
    private java.util.HashMap<String, Node> _contents = new java.util.HashMap<String, Node>();
    private boolean _destroyed = false;
}
